REM Copyright by Victor Aguilar, all rights reserved
REM Program written in QBASIC under MS-DOS
DECLARE FUNCTION equal% (x, y)
DECLARE FUNCTION area! (z) : DECLARE SUB fail (msg$)
COMMON SHARED black%, blue%, green%, cyan%, red%, magenta%, brown%, white%
COMMON SHARED big%, top%, bottom%, left%, right%, tolerance
black% = 0: blue% = 1: green% = 2: cyan% = 3
red% = 4: magenta% = 5: brown% = 6: white% = 7
top% = 330: bottom% = -150: left% = -10: right% = 630
big% = 999: tries% = 99: DIM idu(big%), p(tries%), c%(right%)
SCREEN 12: VIEW:  ' Must use a VGA monitor - - 640X480, 16 color
WINDOW (left%, top%)-(right%, bottom%): COLOR cyan%

REM Input coefficients, find diminishing utility, and estimate the price
CLS : LOCATE 23, 4: PRINT "Quantify the following:"
INPUT "Importance of the phenomenon relative to money (1)          "; mu
INPUT "Difficulty of substituting other phenomena (.25)                    "; sg
IF mu < 0 THEN fail "Importance must be non-negative."
IF sg <= 0 THEN fail "Substitutability must be positive."
isg = 1 / sg
INPUT "Common ratio for an exponentially diminishing utility (.5)  "; k
COLOR white% + 8: tolerance = .000001 ' Relative accuracy of output
DEF fnu (s) = EXP(LOG(k) * s)
u0 = fnu(0): idu(0) = 1: g = 1
FOR n% = 1 TO big%
	un = fnu(n%): t = u: u = u + un: idu(n%) = u0 / un
	gn = idu(n% - 1) / idu(n%): g = g * gn
	IF gn >= 1 THEN fail "Common ratio must always be less than one."
	IF gn <= 0 THEN fail "Common ratio must always be positive."
	IF t = u THEN EXIT FOR
	NEXT n%: IF n% >= big% THEN fail "Utility did not converge."
e1 = EXP(sg): e2 = EXP(sg * sg)
m(0) = .4476505 - 8.923796E-02 * e1 + .4244496 * e2
m(1) = .4867678 - .0949248 * e1 + .3934986 * e2
m(2) = .5510112 - .1388672 * e1 + .3483012 * e2
m(3) = .3653524 - .0758399 * e1 + .2993362 * e2
g = g ^ (1 / n%): i% = INT(4 * g): p(0) = EXP(mu) * m(3)
IF g < .75 THEN
	p(0) = EXP(mu) * (m(i%) + 4 * (m(i% + 1) - m(i%)) * (g - .25 * i%))
END IF

REM Find the price, density, and stock at saturation
FOR i% = 1 TO tries%
	f = 0: f1 = 0: f2 = 0
	FOR m% = 0 TO n%
		x = p(i% - 1) * idu(m%): z = (LOG(x) - mu) * isg
		z2 = z * z: c0 = EXP(-.5 * z2) / (x * sg): t = c: c = c + c0
		f1 = f1 + c0 * (z * isg - 1)
		f2 = f2 + c0 * (1 - z2 * isg * isg + isg * isg)
		IF t = c AND x >= EXP(mu + sg * sg) THEN EXIT FOR
		NEXT m%: IF m% > n% THEN m% = n%
	FOR r% = 0 TO m%
		z = (LOG(p(i% - 1) * idu(r%)) - mu) * isg
		f = f + area(z) / idu(r%): NEXT r%
	f = f - p(i% - 1) * c: h = f / (f1 - f * f2 / (2 * f1 * p(i% - 1)))
	p(i%) = p(i% - 1) - h: c = 0
	IF ABS(h / p(i%)) < tolerance THEN p = p(i%): EXIT FOR
	NEXT i%: IF i% >= tries% THEN fail "Newtons method did not converge."
FOR r% = 0 TO m%
	x = p * idu(r%): z = (LOG(x) - mu) * isg
	c = c + EXP(-.5 * z * z) / (x * sg): NEXT r%: pc = p * c

REM Fill c% array, the demand distribution.
PRINT "The price and stock at saturation are"; p; "and"; pc
scale = 256 ' Width of rectangle (height is one quarter of this.)
COLOR red% + 8
FOR p% = 1 TO right%: p0 = p% * p / scale: d = 0: f = 0
	LOCATE 28, 1: PRINT TAB(4); "percentage done:";
	PRINT USING "###%"; 100 * (p% / right%)
	FOR m% = 0 TO n%
		x = p0 * idu(m%): z = (LOG(x) - mu) * isg
		d0 = EXP(-.5 * z * z) / (x * sg): t = d: d = d + d0
		IF t = d AND x >= EXP(mu + sg * sg) THEN EXIT FOR
		NEXT m%: IF m% > n% THEN m% = n%
	FOR r% = 0 TO m%
		z = (LOG(p0 * idu(r%)) - mu) * isg
		f = f + area(z) / idu(r%): NEXT r%
		f = f - p0 * d
	IF f0 > 0 AND f <= 0 THEN
		max(k%) = p0 - (p * f / scale) / (f / f0): k% = k% + 1
		END IF
	IF d > c * 128000 / scale THEN
		c%(p%) = 32000
	ELSE
		c%(p%) = INT(.25 * scale * d / c + .5)
	END IF
	f0 = f: NEXT p%

REM List relative maximas and check assertion made in Appendix B
COLOR white% + 8: LOCATE 28, 1
PRINT "Aggregate utility is maximized near";
FOR k% = 0 TO 9
	PRINT max(k%); : b = EXP(mu - sg * sg * SQR(1 + isg * isg))
	IF max(k%) > b THEN max% = max% + 1
	IF max(k% + 1) > 0 THEN PRINT "and";  ELSE EXIT FOR
	NEXT k%: IF max(1) > 0 THEN PRINT  ELSE PRINT "only."
IF max% = 1 THEN
	PRINT "There is one relative maxima greater than"; b;
ELSE
	PRINT "There are"; max%; "relative maximas greater than"; b;
END IF

REM Print graph, axis, rectangle, etc.
FOR x% = 2 TO right%
	p% = x% - 1: LINE (p%, c%(p%))-(x%, c%(x%)), magenta% + 8
	NEXT x%
caxis% = green% + 8
LINE (0, top%)-(0, left%), caxis%   'Y axis doesn't go to bottom%
LINE (left%, 0)-(right%, 0), caxis%  'X axis
csat% = brown% + 8: xsat% = scale: ysat% = scale / 4  'saturation
LINE (0, ysat%)-(xsat%, ysat%), csat%
LINE (xsat%, 0)-(xsat%, ysat%), csat%
COLOR white% + 8: LOCATE 22, 29: PRINT p: LOCATE 20, 36: PRINT pc
COLOR red% + 8: LOCATE 2, 50: PRINT "Demand Distribution for"
COLOR brown% + 8: LOCATE 3, 50: PRINT "Axiomatic Theory of Economics"
COLOR red% + 8: LOCATE 4, 50: PRINT "by Victor Aguilar"
COLOR cyan%
END

FUNCTION area (z)
z2 = z * z: y = EXP(-.5 * z2): halfarea = 1.2533141373155#
IF ABS(z) < tolerance THEN
	a = halfarea
ELSE
	SELECT CASE z
		CASE IS > 1.28
			GOSUB contfrac
		CASE 0 TO 1.28
			GOSUB summation: a = halfarea - w
		CASE -1.28 TO 0
			z = -z: GOSUB summation: a = halfarea + w
		CASE IS < -1.28
			z = -z: GOSUB contfrac: a = 2 * halfarea - a
	END SELECT
END IF
area = a: EXIT FUNCTION

contfrac:
w = z * y: t = w
u = z: v = z2 + 1: s = y / u: m = s: a = w / v
FOR j% = 2 TO big%
	s = z * w + j% * y: y = w: w = s
	s = z * v + j% * u: u = v: v = s: s = m: m = a: a = w / v
	IF equal%(m, a) OR equal%(s, a) THEN EXIT FOR
	NEXT j%: IF j% >= big% THEN fail "Contfrac did not converge"
RETURN

summation:
w = z * y: t = w
FOR j% = 3 TO big% STEP 2
	t = t * z2 / j%: s = w: w = w + t
	IF equal%(w, s) THEN EXIT FOR
	NEXT j%: IF j% >= big% THEN fail "Summation did not converge"
RETURN

END FUNCTION

FUNCTION equal% (x, y)
IF x + y THEN
	equal% = ABS(2 * (x - y) / (x + y)) < tolerance
ELSE
	equal% = x = 0  'either x,y = 0 or x = -y
END IF
END FUNCTION

SUB fail (msg$)
CLS : LOCATE 15, 20: COLOR brown% + 8
PRINT "ERROR: "; msg$
END
END SUB

